@@ -18,7 +18,7 @@ from lxml import etree, html
 # own
 from .extractors import (discard_unwanted, extract_url_date,
                          extract_partial_url_date, idiosyncrasies_search,
-                         json_search, timestamp_search, try_ymd_date,
+                         img_search, json_search, timestamp_search, try_ymd_date,
                          ADDITIONAL_EXPRESSIONS, DATE_EXPRESSIONS,
                          YEAR_PATTERN, YMD_PATTERN, COPYRIGHT_PATTERN,
                          THREE_PATTERN, THREE_CATCH,
@@ -35,14 +35,13 @@ from .validators import (check_extracted_reference, compare_values,
                          get_min_date, get_max_date, output_format_validator,
                          plausible_year_filter)
 
-# TODO:
-# from og:image or <img>?
-# time-ago datetime= relative-time datetime=
-# German/English switch
 
 LOGGER = logging.getLogger(__name__)
+def logstring(element):
+    '''Format the element to be logged to a string.'''
+    return html.tostring(element, pretty_print=False, encoding='unicode').strip()
 
-NAME_ATTRIBUTES = set([
+DATE_ATTRIBUTES = set([
                   'article.created', 'article_date_original',
                   'article.published', 'created', 'cxenseparse:recs:publishtime',
                   'date', 'date_published', 'dc.date', 'dc.date.created',
@@ -50,15 +49,14 @@ NAME_ATTRIBUTES = set([
                   'originalpublicationdate', 'parsely-pub-date',
                   'pubdate', 'publishdate', 'publish_date',
                   'published-date', 'publication_date', 'sailthru.date',
-                  'timestamp'
-                   ])
-PROPERTY_ATTRIBUTES = set([
-                      'article:published_time', 'bt:pubdate', 'dc:created',
-                      'dc:date', 'og:article:published_time',
-                      'og:published_time', 'sailthru.date', 'rnews:datepublished'
-                      ])
+                  'timestamp',
+                  'article:published_time', 'bt:pubdate', 'datecreated',
+                  'dateposted', 'datepublished', 'dc:created', 'dc:date',
+                  'og:article:published_time', 'og:published_time',
+                  'sailthru.date', 'rnews:datepublished'
+                  ])
 PROPERTY_MODIFIED = set([
-                    'article:modified_time', 'modified_time',
+                    'article:modified_time', 'datemodified', 'modified_time',
                     'og:article:modified_time', 'og:updated_time',
                     'release_date', 'updated_time'
                     ])
@@ -75,27 +73,34 @@ def examine_date_elements(tree, expression, outputformat, extensive_search, min_
     if not elements or len(elements) > MAX_POSSIBLE_CANDIDATES:
         return None
     # loop through the elements to analyze
+    attempt = None
     for elem in elements:
         # trim
         textcontent = re.sub(r'[\n\r\s\t]+', ' ', elem.text_content(), re.MULTILINE).strip()
         # simple length heuristics
-        if not textcontent or len(textcontent) < 6:
-            continue
-        # shorten and try the beginning of the string
-        # trim non-digits at the end of the string
-        toexamine = re.sub(r'\D+$', '', textcontent[:48])
-        # more than 4 digits required # list(filter(str.isdigit, toexamine))
-        if len([c for c in toexamine if c.isdigit()]) < 4:
-            continue
-        LOGGER.debug('analyzing (HTML): %s', html.tostring(
-            elem, pretty_print=False, encoding='unicode').translate(
-                {ord(c): None for c in '\n\t\r'}
-            ).strip()[:100])
-        attempt = try_ymd_date(toexamine, outputformat, extensive_search, min_date, max_date)
-        if attempt is not None:
-            return attempt
+        if textcontent is not None and len(textcontent) > 6:
+            # shorten and try the beginning of the string
+            # trim non-digits at the end of the string
+            toexamine = re.sub(r'\D+$', '', textcontent[:48])
+            # more than 4 digits required # list(filter(str.isdigit, toexamine))
+            #if len([c for c in toexamine if c.isdigit()]) < 4:
+            #    continue
+            LOGGER.debug('analyzing (HTML): %s', html.tostring(
+                elem, pretty_print=False, encoding='unicode').translate(
+                    {ord(c): None for c in '\n\t\r'}
+                ).strip()[:100])
+            attempt = try_ymd_date(toexamine, outputformat, extensive_search, min_date, max_date)
+            if attempt is not None:
+                break
+        # try link title (Blogspot)
+        title_attr = elem.get('title')
+        if title_attr is not None and len(title_attr) > 0:
+            toexamine = re.sub(r'\D+$', '', title_attr[:48])
+            attempt = try_ymd_date(toexamine, outputformat, extensive_search, min_date, max_date)
+            if attempt is not None:
+                break
     # catchall
-    return None
+    return attempt
 
 
 def examine_header(tree, outputformat, extensive_search, original_date, min_date, max_date):
@@ -134,73 +139,77 @@ def examine_header(tree, outputformat, extensive_search, original_date, min_date
             continue
         # property attribute
         if 'property' in elem.attrib and 'content' in elem.attrib:
+            attribute = elem.get('property').lower()
             # original date
             if original_date is True:
-                if elem.get('property').lower() in PROPERTY_ATTRIBUTES:
-                    LOGGER.debug('examining meta property: %s', html.tostring(elem, pretty_print=False, encoding='unicode').strip())
+                if attribute in DATE_ATTRIBUTES:
+                    LOGGER.debug('examining meta property: %s', logstring(elem))
                     headerdate = tryfunc(elem.get('content'))
             # modified date: override published_time
             else:
-                if elem.get('property').lower() in PROPERTY_MODIFIED:
-                    LOGGER.debug('examining meta property: %s', html.tostring(elem, pretty_print=False, encoding='unicode').strip())
-                    attempt = tryfunc(elem.get('content'))
-                    if attempt is not None:
-                        headerdate = attempt
-                elif elem.get('property').lower() in PROPERTY_ATTRIBUTES \
-                    and headerdate is None:
-                    LOGGER.debug('examining meta property: %s', html.tostring(elem, pretty_print=False, encoding='unicode').strip())
+                if attribute in PROPERTY_MODIFIED:
+                    LOGGER.debug('examining meta property: %s', logstring(elem))
+                    headerdate = tryfunc(elem.get('content'))
+                elif attribute in DATE_ATTRIBUTES:
+                    LOGGER.debug('examining meta property: %s', logstring(elem))
                     headerdate = tryfunc(elem.get('content'))
         # name attribute
-        elif headerdate is None and 'name' in elem.attrib and 'content' in elem.attrib:
+        elif 'name' in elem.attrib and 'content' in elem.attrib:
             # url
             if elem.get('name').lower() == 'og:url':
                 headerdate = extract_url_date(elem.get('content'), outputformat)
             # date
-            elif elem.get('name').lower() in NAME_ATTRIBUTES:
-                LOGGER.debug('examining meta name: %s', html.tostring(elem, pretty_print=False, encoding='unicode').strip())
+            elif elem.get('name').lower() in DATE_ATTRIBUTES:
+                LOGGER.debug('examining meta name: %s', logstring(elem))
                 headerdate = tryfunc(elem.get('content'))
             # modified
-            elif elem.get('name').lower() in ('lastmodified', 'last-modified') and original_date is False:
-                LOGGER.debug('examining meta name: %s', html.tostring(elem, pretty_print=False, encoding='unicode').strip())
-                headerdate = tryfunc(elem.get('content'))
-        elif headerdate is None and 'pubdate' in elem.attrib:
+            elif elem.get('name').lower() in ('lastmodified', 'last-modified'):
+                LOGGER.debug('examining meta name: %s', logstring(elem))
+                if original_date is False:
+                    headerdate = tryfunc(elem.get('content'))
+                else:
+                    reserve = tryfunc(elem.get('content'))
+        elif 'pubdate' in elem.attrib:
             if elem.get('pubdate').lower() == 'pubdate':
-                LOGGER.debug('examining meta pubdate: %s', html.tostring(elem, pretty_print=False, encoding='unicode').strip())
+                LOGGER.debug('examining meta pubdate: %s', logstring(elem))
                 headerdate = tryfunc(elem.get('content'))
         # other types # itemscope?
-        elif headerdate is None and 'itemprop' in elem.attrib:
-            if elem.get('itemprop').lower() in (
+        elif 'itemprop' in elem.attrib:
+            attribute = elem.get('itemprop').lower()
+            if attribute in (
                     'datecreated', 'datepublished', 'pubyear'
-                ) and headerdate is None:
-                LOGGER.debug('examining meta itemprop: %s', html.tostring(elem, pretty_print=False, encoding='unicode').strip())
+                ):
+                LOGGER.debug('examining meta itemprop: %s', logstring(elem))
                 if 'datetime' in elem.attrib:
                     headerdate = try_ymd_date(elem.get('datetime'), outputformat, extensive_search, min_date, max_date)
                 elif 'content' in elem.attrib:
                     headerdate = tryfunc(elem.get('content'))
             # override
-            elif elem.get('itemprop').lower() == 'datemodified' and original_date is False:
-                LOGGER.debug('examining meta itemprop: %s', html.tostring(elem, pretty_print=False, encoding='unicode').strip())
+            elif attribute == 'datemodified' and original_date is False:
+                LOGGER.debug('examining meta itemprop: %s', logstring(elem))
                 if 'datetime' in elem.attrib:
-                    attempt = try_ymd_date(elem.get('datetime'), outputformat, extensive_search, min_date, max_date)
+                    headerdate = try_ymd_date(elem.get('datetime'), outputformat, extensive_search, min_date, max_date)
                 elif 'content' in elem.attrib:
-                    attempt = tryfunc(elem.get('content'))
-                if attempt is not None:
-                    headerdate = attempt
+                    headerdate = tryfunc(elem.get('content'))
             # reserve with copyrightyear
-            elif headerdate is None and elem.get('itemprop').lower() == 'copyrightyear':
-                LOGGER.debug('examining meta itemprop: %s', html.tostring(elem, pretty_print=False, encoding='unicode').strip())
+            elif attribute == 'copyrightyear':
+                LOGGER.debug('examining meta itemprop: %s', logstring(elem))
                 if 'content' in elem.attrib:
                     attempt = '-'.join([elem.get('content'), '01', '01'])
                     if date_validator(attempt, '%Y-%m-%d', latest=max_date) is True:
                         reserve = attempt
         # http-equiv, rare http://www.standardista.com/html5/http-equiv-the-meta-attribute-explained/
-        elif headerdate is None and 'http-equiv' in elem.attrib:
-            if original_date is True and elem.get('http-equiv').lower() == 'date':
-                LOGGER.debug('examining meta http-equiv: %s', html.tostring(elem, pretty_print=False, encoding='unicode').strip())
-                headerdate = tryfunc(elem.get('content'))
-            if elem.get('http-equiv').lower() in ('date', 'last-modified'):
-                LOGGER.debug('examining meta http-equiv: %s', html.tostring(elem, pretty_print=False, encoding='unicode').strip())
+        elif 'http-equiv' in elem.attrib:
+            attribute = elem.get('http-equiv').lower()
+            if attribute == 'date':
+                LOGGER.debug('examining meta http-equiv: %s', logstring(elem))
                 headerdate = tryfunc(elem.get('content'))
+            elif attribute == 'last-modified':
+                LOGGER.debug('examining meta http-equiv: %s', logstring(elem))
+                if original_date is False:
+                    headerdate = tryfunc(elem.get('content'))
+                else:
+                    reserve = tryfunc(elem.get('content'))
         # exit loop
         if headerdate is not None:
             break
@@ -243,16 +252,16 @@ def select_candidate(occurrences, catch, yearpat, original_date, min_date, max_d
         if date_validator(str(year1), '%Y', earliest=min_date, latest=max_date) is False:
             if date_validator(str(year2), '%Y', earliest=min_date, latest=max_date) is True:
                 # LOGGER.debug('first candidate not suitable: %s', year1)
-                match = catch.match(second_pattern)
+                match = catch.search(second_pattern)
             else:
                 LOGGER.debug('no suitable candidate: %s %s', year1, year2)
                 return None
         # safety net: newer date but up to 50% less frequent
         if year2 != year1 and second_count/first_count > 0.5:
-            match = catch.match(second_pattern)
+            match = catch.search(second_pattern)
         # not newer or hopefully not significant
         else:
-            match = catch.match(first_pattern)
+            match = catch.search(first_pattern)
     return match
 
 
@@ -270,8 +279,7 @@ def try_expression(expression, outputformat, extensive_search, min_date, max_dat
     if not textcontent or len([c for c in textcontent if c.isdigit()]) < 4:
         return None
     # try the beginning of the string
-    attempt = try_ymd_date(textcontent[:48], outputformat, extensive_search, min_date, max_date)
-    return attempt
+    return try_ymd_date(textcontent[:48], outputformat, extensive_search, min_date, max_date)
 
 
 @lru_cache(maxsize=32)
@@ -390,6 +398,19 @@ def examine_time_elements(tree, outputformat, extensive_search, original_date, m
     return None
 
 
+def normalize_match(match):
+    '''Normalize string output by adding "0" if necessary.'''
+    if len(match.group(1)) == 1:
+        day = '0' + match.group(1)
+    else:
+        day = match.group(1)
+    if len(match.group(2)) == 1:
+        month = '0' + match.group(2)
+    else:
+        month = match.group(2)
+    return day, month
+
+
 def search_page(htmlstring, outputformat, original_date, min_date, max_date):
     """
     Opportunistically search the HTML text for common text patterns
@@ -409,9 +430,6 @@ def search_page(htmlstring, outputformat, original_date, min_date, max_date):
     :return: Returns a valid date expression as a string, or None
 
     """
-    # init
-    # © Janssen-Cilag GmbH 2014-2019. https://www.krebsratgeber.de/artikel/was-macht-eine-zelle-zur-krebszelle
-    # date ultimate rescue for the rest: most frequent year/month comination in the HTML
 
     # copyright symbol
     LOGGER.debug('looking for copyright/footer information')
@@ -419,23 +437,21 @@ def search_page(htmlstring, outputformat, original_date, min_date, max_date):
     bestmatch = search_pattern(htmlstring, COPYRIGHT_PATTERN, YEAR_PATTERN, YEAR_PATTERN, original_date, min_date, max_date)
     if bestmatch is not None:
         LOGGER.debug('Copyright detected: %s', bestmatch.group(0))
-        pagedate = '-'.join([bestmatch.group(0), '07', '01'])
         if date_validator(bestmatch.group(0), '%Y', latest=max_date) is True:
-            LOGGER.debug('date found for copyright/footer pattern "%s": %s', COPYRIGHT_PATTERN, pagedate)
+            LOGGER.debug('copyright year/footer pattern found: %s', bestmatch.group(0))
             copyear = int(bestmatch.group(0))
-            # return convert_date(pagedate, '%Y-%m-%d', outputformat)
 
     # 3 components
     LOGGER.debug('3 components')
     # target URL characteristics
     bestmatch = search_pattern(htmlstring, THREE_PATTERN, THREE_CATCH, YEAR_PATTERN, original_date, min_date, max_date)
-    result = filter_ymd_candidate(bestmatch, THREE_PATTERN, copyear, outputformat, min_date, max_date)
+    result = filter_ymd_candidate(bestmatch, THREE_PATTERN, original_date, copyear, outputformat, min_date, max_date)
     if result is not None:
         return result
 
     # more loosely structured data
     bestmatch = search_pattern(htmlstring, THREE_LOOSE_PATTERN, THREE_LOOSE_CATCH, YEAR_PATTERN, original_date, min_date, max_date)
-    result = filter_ymd_candidate(bestmatch, THREE_LOOSE_PATTERN, copyear, outputformat, min_date, max_date)
+    result = filter_ymd_candidate(bestmatch, THREE_LOOSE_PATTERN, original_date, copyear, outputformat, min_date, max_date)
     if result is not None:
         return result
 
@@ -445,26 +461,19 @@ def search_page(htmlstring, outputformat, original_date, min_date, max_date):
     replacement = dict()
     for item in candidates:
         match = re.match(r'([0-3]?[0-9])[/.-]([01]?[0-9])[/.-]([0-9]{4})', item)
-        if len(match.group(1)) == 1:
-            day = '0' + match.group(1)
-        else:
-            day = match.group(1)
-        if len(match.group(2)) == 1:
-            month = '0' + match.group(2)
-        else:
-            month = match.group(2)
+        day, month = normalize_match(match)
         candidate = '-'.join([match.group(3), month, day])
         replacement[candidate] = candidates[item]
     candidates = Counter(replacement)
     # select
     bestmatch = select_candidate(candidates, YMD_PATTERN, YMD_YEAR, original_date, min_date, max_date)
-    result = filter_ymd_candidate(bestmatch, SELECT_YMD_PATTERN, copyear, outputformat, min_date, max_date)
+    result = filter_ymd_candidate(bestmatch, SELECT_YMD_PATTERN, original_date, copyear, outputformat, min_date, max_date)
     if result is not None:
         return result
 
     # valid dates strings
     bestmatch = search_pattern(htmlstring, DATESTRINGS_PATTERN, DATESTRINGS_CATCH, YEAR_PATTERN, original_date, min_date, max_date)
-    result = filter_ymd_candidate(bestmatch, DATESTRINGS_PATTERN, copyear, outputformat, min_date, max_date)
+    result = filter_ymd_candidate(bestmatch, DATESTRINGS_PATTERN, original_date, copyear, outputformat, min_date, max_date)
     if result is not None:
         return result
 
@@ -474,14 +483,7 @@ def search_page(htmlstring, outputformat, original_date, min_date, max_date):
     replacement = dict()
     for item in candidates:
         match = re.match(r'([0-3]?[0-9])[/.]([01]?[0-9])[/.]([0-9]{2})', item)
-        if len(match.group(1)) == 1:
-            day = '0' + match.group(1)
-        else:
-            day = match.group(1)
-        if len(match.group(2)) == 1:
-            month = '0' + match.group(2)
-        else:
-            month = match.group(2)
+        day, month = normalize_match(match)
         if match.group(3)[0] == '9':
             year = '19' + match.group(3)
         else:
@@ -490,7 +492,7 @@ def search_page(htmlstring, outputformat, original_date, min_date, max_date):
         replacement[candidate] = candidates[item]
     candidates = Counter(replacement)
     bestmatch = select_candidate(candidates, YMD_PATTERN, YMD_YEAR, original_date, min_date, max_date)
-    result = filter_ymd_candidate(bestmatch, SLASHES_PATTERN, copyear, outputformat, min_date, max_date)
+    result = filter_ymd_candidate(bestmatch, SLASHES_PATTERN, original_date, copyear, outputformat, min_date, max_date)
     if result is not None:
         return result
 
@@ -504,6 +506,7 @@ def search_page(htmlstring, outputformat, original_date, min_date, max_date):
             if copyear == 0 or int(bestmatch.group(1)) >= copyear:
                 LOGGER.debug('date found for pattern "%s": %s', YYYYMM_PATTERN, pagedate)
                 return convert_date(pagedate, '%Y-%m-%d', outputformat)
+
     # 2 components, second option
     candidates = plausible_year_filter(htmlstring, MMYYYY_PATTERN, MMYYYY_YEAR, original_date)
     # revert DD-MM-YYYY patterns before sorting
@@ -519,10 +522,15 @@ def search_page(htmlstring, outputformat, original_date, min_date, max_date):
     candidates = Counter(replacement)
     # select
     bestmatch = select_candidate(candidates, YMD_PATTERN, YMD_YEAR, original_date, min_date, max_date)
-    result = filter_ymd_candidate(bestmatch, MMYYYY_PATTERN, copyear, outputformat, min_date, max_date)
+    result = filter_ymd_candidate(bestmatch, MMYYYY_PATTERN, original_date, copyear, outputformat, min_date, max_date)
     if result is not None:
         return result
 
+    # catchall
+    if copyear != 0:
+        LOGGER.debug('using copyright year as default')
+        return convert_date('-'.join([str(copyear), '01', '01']), '%Y-%m-%d', outputformat)
+
     # 1 component, last try
     LOGGER.debug('switching to one component')
     bestmatch = search_pattern(htmlstring, SIMPLE_PATTERN, YEAR_PATTERN, YEAR_PATTERN, original_date, min_date, max_date)
@@ -533,10 +541,6 @@ def search_page(htmlstring, outputformat, original_date, min_date, max_date):
                 LOGGER.debug('date found for pattern "%s": %s', SIMPLE_PATTERN, pagedate)
                 return convert_date(pagedate, '%Y-%m-%d', outputformat)
 
-    # catchall
-    if copyear != 0:
-        pagedate = '-'.join([str(copyear), '01', '01'])
-        return convert_date(pagedate, '%Y-%m-%d', outputformat)
     return None
 
 
@@ -580,8 +584,7 @@ def find_date(htmlobject, extensive_search=True, original_date=False, outputform
         logging.basicConfig(level=logging.DEBUG)
     tree = load_html(htmlobject)
     find_date.extensive_search = extensive_search
-    min_date = get_min_date(min_date)
-    max_date = get_max_date(max_date)
+    min_date, max_date = get_min_date(min_date), get_max_date(max_date)
 
     # safety
     if tree is None:
@@ -681,16 +684,38 @@ def find_date(htmlobject, extensive_search=True, original_date=False, outputform
     if text_result is not None:
         return text_result
 
+    # title
+    for title_elem in cleaned_html.iterfind('.//title|.//h1'):
+        attempt = try_ymd_date(title_elem.text_content(), outputformat, extensive_search, min_date, max_date)
+        if attempt is not None:
+            return attempt
+
     # last try: URL 2
     if url is not None:
         dateresult = extract_partial_url_date(url, outputformat)
         if dateresult is not None:
             return dateresult
 
+    # try image elements
+    img_result = img_search(
+        tree, outputformat, min_date, max_date
+        )
+    if img_result is not None:
+        return img_result
+
     # last resort
     if extensive_search is True:
         LOGGER.debug('extensive search started')
-        pagedate = search_page(htmlstring, outputformat, original_date, min_date, max_date)
-        return pagedate
+        # div and p elements?
+        # TODO: check all and decide according to original_date
+        reference = 0
+        for textpart in [t for t in cleaned_html.xpath('.//div/text()|.//p/text()') if 0 < len(t) < 80]:
+            reference = compare_reference(reference, textpart, outputformat, extensive_search, original_date, min_date, max_date)
+        # return
+        converted = check_extracted_reference(reference, outputformat, min_date, max_date)
+        if converted is not None:
+            return converted
+        # search page HTML
+        return search_page(htmlstring, outputformat, original_date, min_date, max_date)
 
     return None
