@@ -145,17 +145,17 @@ TEXT_MONTHS = {'Januar': '01', 'Jänner': '01', 'January': '01', 'Jan': '01',
                'Aralık': '12', 'Ara': '12'}
 
 TEXT_DATE_PATTERN = re.compile(r'[.:,_/ -]|^[0-9]+$')
-NO_TEXT_DATE_PATTERN = re.compile(r'[0-9]{3,}-[0-9]{3,}|[0-9]{2}:[0-9]{2}(:| )|\D*[0-9]{4}\D*$')
+NO_TEXT_DATE_PATTERN = re.compile(r'[0-9]{3,}\D+[0-9]{3,}|[0-9]{2}:[0-9]{2}(:| )|\D*[0-9]{4}\D*$')
 
 # use of regex module for speed
-EN_PATTERNS = regex.compile(r'(?:updated|published) *?(?:in)? *?:? *?([0-9]{1,2})[./]([0-9]{1,2})[./]([0-9]{2,4})', re.I)
+EN_PATTERNS = regex.compile(r'(?:[Dd]ate[^0-9"]{,20}|updated|published) *?(?:in)? *?:? *?([0-9]{1,4})[./]([0-9]{1,2})[./]([0-9]{2,4})', re.I)
 DE_PATTERNS = regex.compile(r'(?:Datum|Stand): ?([0-9]{1,2})\.([0-9]{1,2})\.([0-9]{2,4})')
 TR_PATTERNS = regex.compile(r'''(?:güncellen?me|yayı(?:m|n)lan?ma) *?(?:tarihi)? *?:? *?([0-9]{1,2})[./]([0-9]{1,2})[./]([0-9]{2,4})|
 ([0-9]{1,2})[./]([0-9]{1,2})[./]([0-9]{2,4}) *?(?:'de|'da|'te|'ta|’de|’da|’te|’ta|tarihinde) *(?:güncellendi|yayı(?:m|n)landı)'''.replace('\n', ''), re.I)
 
 # extensive search patterns
 YEAR_PATTERN = re.compile(r'^\D?(199[0-9]|20[0-9]{2})')
-COPYRIGHT_PATTERN = re.compile(r'(?:©|\&copy;|Copyright|\(c\))\D*([12][0-9]{3})\D')
+COPYRIGHT_PATTERN = re.compile(r'(?:©|\&copy;|Copyright|\(c\))\D*(?:[12][0-9]{3}-)?([12][0-9]{3})\D')
 THREE_PATTERN = re.compile(r'/([0-9]{4}/[0-9]{2}/[0-9]{2})[01/]')
 THREE_CATCH = re.compile(r'([0-9]{4})/([0-9]{2})/([0-9]{2})')
 THREE_LOOSE_PATTERN = re.compile(r'\D([0-9]{4}[/.-][0-9]{2}[/.-][0-9]{2})\D')
@@ -307,7 +307,7 @@ def custom_parse(string, outputformat, extensive_search, min_date, max_date):
             if date_validator(result, outputformat, earliest=min_date, latest=max_date) is True:
                 LOGGER.debug('parsing result: %s', result)
                 return result.strftime(outputformat)
-        except (OverflowError, ValueError):
+        except (OverflowError, TypeError, ValueError):
             LOGGER.debug('parsing error: %s', string)
     # %Y-%m-%d search
     match = YMD_PATTERN.search(string)
@@ -344,6 +344,7 @@ def custom_parse(string, outputformat, extensive_search, min_date, max_date):
     # text match
     dateobject = regex_parse(string)
     # copyright match?
+    #if dateobject is None:
     # © Janssen-Cilag GmbH 2014-2019. https://www.krebsratgeber.de/artikel/was-macht-eine-zelle-zur-krebszelle
     # examine
     if dateobject is not None:
@@ -379,11 +380,13 @@ def try_ymd_date(string, outputformat, extensive_search, min_date, max_date):
     """Use a series of heuristics and rules to parse a potential date expression"""
     # discard on formal criteria
     # list(filter(str.isdigit, string))
-    if not string or len(string) < 6 or len([c for c in string if c.isdigit()]) < 4 \
-    or not TEXT_DATE_PATTERN.search(string):
+    if not string or len(string) < 6:
         return None
-    # just time/single year, not a date
-    if NO_TEXT_DATE_PATTERN.match(string):
+    digits_num = len([c for c in string if c.isdigit()])
+    if not 4 <= digits_num <= 18:
+        return None
+    # just time/single year or digits, not a date
+    if not TEXT_DATE_PATTERN.search(string) or NO_TEXT_DATE_PATTERN.match(string):
         return None
     # faster
     customresult = custom_parse(string, outputformat, extensive_search, min_date, max_date)
@@ -399,6 +402,16 @@ def try_ymd_date(string, outputformat, extensive_search, min_date, max_date):
     return None
 
 
+def img_search(tree, outputformat, min_date, max_date):
+    '''Skim through image elements'''
+    element = tree.find('.//meta[@property="og:image"]')
+    if element is not None and 'content' in element.attrib:
+        result = extract_url_date(element.get('content'), outputformat)
+        if result is not None and date_validator(result, outputformat, earliest=min_date, latest=max_date) is True:
+            return result
+    return None
+
+
 def json_search(tree, outputformat, original_date, min_date, max_date):
     '''Look for JSON time patterns in JSON sections of the tree'''
     # determine pattern
@@ -428,6 +441,7 @@ def timestamp_search(htmlstring, outputformat, min_date, max_date):
 
 def extract_idiosyncrasy(idiosyncrasy, htmlstring, outputformat, min_date, max_date):
     '''Look for a precise pattern throughout the web page'''
+    candidate = None
     match = idiosyncrasy.search(htmlstring)
     groups = [0, 1, 2, 3] if match and match.group(3) else [] #because len(None) has no len
     try:
@@ -435,7 +449,16 @@ def extract_idiosyncrasy(idiosyncrasy, htmlstring, outputformat, min_date, max_d
     except IndexError:
         pass
     if match and groups: #because len(None) has no len
-        if len(match.group(groups[3])) in (2, 4):
+        if match.group(1) is not None and len(match.group(1)) == 4:
+            candidate = datetime.date(int(match.group(groups[1])),
+                                      int(match.group(groups[2])),
+                                      int(match.group(groups[3])))
+        elif len(match.group(groups[3])) in (2, 4):
+            # switch to MM/DD/YY
+            if int(match.group(groups[2])) > 12:
+                tmp1, tmp2 = groups[1], groups[2]
+                groups[1], groups[2] = tmp2, tmp1
+            # DD/MM/YY
             try:
                 if len(match.group(groups[3])) == 2:
                     candidate = datetime.date(int('20' + match.group(groups[3])),
@@ -447,10 +470,10 @@ def extract_idiosyncrasy(idiosyncrasy, htmlstring, outputformat, min_date, max_d
                                               int(match.group(groups[1])))
             except ValueError:
                 LOGGER.debug('value error in idiosyncrasies: %s', match.group(0))
-            else:
-                if date_validator(candidate, '%Y-%m-%d', earliest=min_date, latest=max_date) is True:
-                    LOGGER.debug('idiosyncratic pattern found: %s', match.group(0))
-                    return convert_date(candidate, '%Y-%m-%d', outputformat)
+    if candidate is not None:
+        if date_validator(candidate, '%Y-%m-%d', earliest=min_date, latest=max_date) is True:
+            LOGGER.debug('idiosyncratic pattern found: %s', match.group(0))
+            return convert_date(candidate, '%Y-%m-%d', outputformat)
     return None
 
 
