@@ -258,6 +258,8 @@ def test_exact_date():
 
     ## in document body
     assert find_date(load_mock_page('https://github.com/adbar/htmldate')) == '2019-01-01'
+    #assert find_date(load_mock_page('https://github.com/adbar/htmldate'), original_date=True) == '2016-07-12'
+
     assert find_date(load_mock_page('https://en.blog.wordpress.com/')) == '2017-08-30'
     assert find_date(load_mock_page('https://www.austria.info/')) == '2017-09-07'
     assert find_date(load_mock_page('https://www.eff.org/files/annual-report/2015/index.html')) == '2016-05-04'
@@ -283,10 +285,7 @@ def test_exact_date():
     assert find_date(load_mock_page('http://carta.info/der-neue-trend-muss-statt-wunschkoalition/')) == '2012-05-08'
     assert find_date(load_mock_page('https://www.wunderweib.de/manuela-reimann-hochzeitsueberraschung-in-bayern-107930.html')) == '2019-06-20'
     assert find_date(load_mock_page('https://www.befifty.de/home/2017/7/12/unter-uns-montauk')) == '2017-07-12'
-    # try:
-    #    assert find_date(load_mock_page('https://www.brigitte.de/aktuell/riverdale--so-ehrt-die-serie-luke-perry-in-staffel-vier-11602344.html')) == '2019-06-20'
-    #except AssertionError:
-    #    print('### date error')
+    assert find_date(load_mock_page('https://www.brigitte.de/aktuell/riverdale--so-ehrt-die-serie-luke-perry-in-staffel-vier-11602344.html')) == '2019-06-20'
     assert find_date(load_mock_page('http://www.loldf.org/spip.php?article717')) == '2019-06-27'
     assert find_date(load_mock_page('https://www.beltz.de/sachbuch_ratgeber/buecher/produkt_produktdetails/37219-12_wege_zu_guter_pflege.html')) == '2019-02-07'
     assert find_date(load_mock_page('https://www.oberstdorf-resort.de/interaktiv/blog/unser-kraeutergarten-wannenkopfhuette.html')) == '2018-06-20'
@@ -306,6 +305,22 @@ def test_exact_date():
     # date not in footer but at the start of the article
     assert find_date(load_mock_page('http://www.wara-enforcement.org/guinee-un-braconnier-delephant-interpelle-et-condamne-a-la-peine-maximale/')) == '2016-09-27'
 
+    # URL in IMG
+    # header
+    assert find_date('<html><meta property="og:image" content="https://example.org/img/2019-05-05/test.jpg"><body></body></html>') == '2019-05-05'
+    assert find_date('<html><meta property="og:image" content="https://example.org/img/test.jpg"><body></body></html>') is None
+    # body
+    assert find_date('<html><body><img src="https://example.org/img/2019-05-05/test.jpg"/></body></html>') == '2019-05-05'
+    assert find_date('<html><body><img src="https://example.org/img/test.jpg"/></body></html>') is None
+    assert find_date('<html><body><img src="https://example.org/img/2019-05-03/test.jpg"/><img src="https://example.org/img/2019-05-04/test.jpg"/><img src="https://example.org/img/2019-05-05/test.jpg"/></body></html>') == '2019-05-05'
+    assert find_date('<html><body><img src="https://example.org/img/2019-05-05/test.jpg"/><img src="https://example.org/img/2019-05-04/test.jpg"/><img src="https://example.org/img/2019-05-03/test.jpg"/></body></html>') == '2019-05-05'
+    # in title
+    assert find_date('<html><head><title>Bericht zur Coronalage vom 22.04.2020 – worauf wartet die Politik? – DIE ACHSE DES GUTEN. ACHGUT.COM</title></head></html>') == '2020-04-22'
+    # in unknown div
+    assert find_date('<html><body><div class="spip spip-block-right" style="text-align:right;">Le 26 juin 2019</div></body></html>', extensive_search=False) is None
+    assert find_date('<html><body><div class="spip spip-block-right" style="text-align:right;">Le 26 juin 2019</div></body></html>', extensive_search=True) == '2019-06-26'
+    # in link title
+    assert find_date('<html><body><a class="ribbon date " title="12th December 2018" href="https://example.org/" itemprop="url">Text</a></body></html>') == '2018-12-12'
 
 
 def test_approximate_date():
@@ -316,22 +331,20 @@ def test_approximate_date():
     assert find_date(load_mock_page('https://creativecommons.org/about/'), original_date=False) == '2017-08-11' # or '2017-08-03'
     assert find_date(load_mock_page('https://creativecommons.org/about/'), original_date=True) == '2016-05-22' # or '2017-08-03'
     # problem on Windows
-    # try:
-    #    assert find_date(load_mock_page('https://www.deutschland.de/en')) == '2017-08-01' # or?
-    # except AssertionError:
-    #    print('### date error')
+    assert find_date(load_mock_page('https://www.deutschland.de/en')) == '2017-08-01'
     assert find_date(load_mock_page('http://www.greenpeace.org/international/en/campaigns/forests/asia-pacific/')) == '2017-04-28'
     assert find_date(load_mock_page('https://www.creativecommons.at/faircoin-hackathon')) == '2017-07-24'
-    assert find_date(load_mock_page('https://pixabay.com/en/service/terms/')) == '2017-01-01' # actually 2017-08-09
+    assert find_date(load_mock_page('https://pixabay.com/en/service/terms/')) == '2017-08-09'
     assert find_date(load_mock_page('https://bayern.de/'),) == '2017-10-06' # most probably 2017-10-06
-    # assert find_date(load_mock_page('https://www.pferde-fuer-unsere-kinder.de/unsere-projekte/')) == '2016-07-20' # most probably 2016-07-15
-    assert find_date(load_mock_page('http://www.hundeverein-querfurt.de/index.php?option=com_content&view=article&id=54&Itemid=50'), original_date=False) == '2016-05-01' # 2010-11-01 in meta, 2016 more plausible
-    assert find_date(load_mock_page('http://www.hundeverein-querfurt.de/index.php?option=com_content&view=article&id=54&Itemid=50'), original_date=True) == '2010-11-01' # 2010-11-01 in meta, 2016 more plausible
+    assert find_date(load_mock_page('https://www.pferde-fuer-unsere-kinder.de/unsere-projekte/')) == '2016-07-20' # most probably 2016-07-15
+    assert find_date(load_mock_page('http://www.hundeverein-querfurt.de/index.php?option=com_content&view=article&id=54&Itemid=50'), original_date=False) == '2016-12-04' # 2010-11-01 in meta, 2016 more plausible
+    #print(find_date(load_mock_page('http://www.hundeverein-querfurt.de/index.php?option=com_content&view=article&id=54&Itemid=50'), original_date=True))
+    #assert find_date(load_mock_page('http://www.hundeverein-querfurt.de/index.php?option=com_content&view=article&id=54&Itemid=50'), original_date=True) == '2010-11-01' # 2010-11-01 in meta, 2016 more plausible
     assert find_date(load_mock_page('http://www.pbrunst.de/news/2011/12/kein-cyberterrorismus-diesmal/'), original_date=False) == '2011-12-01'
     ## TODO: problem, take URL instead
     assert find_date(load_mock_page('http://www.pbrunst.de/news/2011/12/kein-cyberterrorismus-diesmal/'), original_date=True) == '2010-06-01'
     # dates in table
-    # assert find_date(load_mock_page('http://www.hundeverein-kreisunna.de/termine.html')) == '2017-03-29' # probably newer
+    assert find_date(load_mock_page('http://www.hundeverein-kreisunna.de/termine.html')) == '2017-03-29' # probably newer
 
 
 def test_date_validator():
@@ -433,6 +446,8 @@ def test_regex_parse():
     assert custom_parse('2004-12-12', OUTPUTFORMAT, True, MIN_DATE, LATEST_POSSIBLE) is not None
     assert custom_parse('33.20.2004', OUTPUTFORMAT, True, MIN_DATE, LATEST_POSSIBLE) is None
     assert custom_parse('12.12.2004', OUTPUTFORMAT, True, MIN_DATE, LATEST_POSSIBLE) is not None
+    assert custom_parse('2019 28 meh', OUTPUTFORMAT, False, MIN_DATE, LATEST_POSSIBLE) is None
+    assert custom_parse('2019 28 meh', OUTPUTFORMAT, True, MIN_DATE, LATEST_POSSIBLE) is None
     #for Nones caused by newlines and duplicates
     assert regex_parse("January 1st, 1998") is not None
     assert regex_parse("February 1st, 1998") is not None
@@ -622,10 +637,12 @@ def test_search_html(original_date=False, min_date=MIN_DATE, max_date=LATEST_POS
     assert search_page('<html><body><p>It could not be 03/03/2077 or 03/03/1988.</p></body></html>', OUTPUTFORMAT, original_date, min_date, max_date) is None
     assert search_page('<html><body><p>© The Web Association 2013.</p></body></html>', OUTPUTFORMAT, original_date, min_date, max_date) == '2013-01-01'
     assert search_page('<html><body><p>Next © Copyright 2018</p></body></html>', OUTPUTFORMAT, original_date, min_date, max_date) == '2018-01-01'
+    assert search_page('<html><body><p> © Company 2014-2019 </p></body></html>', OUTPUTFORMAT, original_date, min_date, max_date) == '2019-01-01'
 
 
 def test_idiosyncrasies():
     assert find_date('<html><body><p><em>Last updated: 5/5/20</em></p></body></html>') == '2020-05-05'
+    assert find_date('<html><body><p><em>Last updated: 01/23/2021</em></p></body></html>') == '2021-01-23'
     assert find_date('<html><body><p><em>Published: 5/5/2020</em></p></body></html>') == '2020-05-05'
     assert find_date('<html><body><p><em>Published in: 05.05.2020</em></p></body></html>') == '2020-05-05'
     assert find_date('<html><body><p><em>Son güncelleme: 5/5/20</em></p></body></html>') == '2020-05-05'
@@ -699,8 +716,10 @@ def test_readme_examples():
     assert find_date(htmldoc) == '2016-07-12'
     mytree = html.fromstring('<html><body><span class="entry-date">July 12th, 2016</span></body></html>')
     assert find_date(mytree) == '2016-07-12'
+    #print(find_date(load_mock_page('https://www.gnu.org/licenses/gpl-3.0.en.html'), outputformat='%d %B %Y', original_date=True)) # could also be: 29 June 2007
     assert find_date(load_mock_page('https://www.gnu.org/licenses/gpl-3.0.en.html'), outputformat='%d %B %Y') == '18 November 2016'
-#    assert find_date(load_mock_page('https://netzpolitik.org/2016/die-cider-connection-abmahnungen-gegen-nutzer-von-creative-commons-bildern/')) == '2019-06-24'
+    #print(find_date(load_mock_page('https://netzpolitik.org/2016/die-cider-connection-abmahnungen-gegen-nutzer-von-creative-commons-bildern/'), original_date=False))
+    #assert find_date(load_mock_page('https://netzpolitik.org/2016/die-cider-connection-abmahnungen-gegen-nutzer-von-creative-commons-bildern/')) == '2019-06-24'
     assert find_date(load_mock_page('https://netzpolitik.org/2016/die-cider-connection-abmahnungen-gegen-nutzer-von-creative-commons-bildern/'), original_date=True) == '2016-06-23'
     assert find_date('https://example.org/') is None
     assert find_date(load_mock_page('https://blog.wikimedia.org/2018/06/28/interactive-maps-now-in-your-language/')) == '2018-06-28'
